﻿unit element;

interface

uses
  System.Generics.Collections;


type
  IElement = interface
    procedure SetName(Value : string);
    function GetName : string;
    property Name : string read GetName write SetName;

    procedure SetNumber(Value : integer);
    function GetNumber : integer;
    property Number : integer read GetNumber write SetNumber;

    procedure SetSymbol(Value : string);
    function GetSymbol : string;
    property Symbol : string read GetSymbol write SetSymbol;

    procedure SetWeight(Value : double);
    function GetWeight : double;
    property Weight : double read GetWeight write SetWeight;
  end;

  TElements = class
  private
    List : TList<IElement>;
  public
    constructor Create;
    destructor Destroy; override;
    procedure LoadFromResource;
    function Element(AtomicNumber : integer) : IElement;
    function Count : integer;
  end;

  TPeriodicTable = class
  private
    TheTable : array[1..18,1..9] of IElement;
  public
    constructor Create(Elements : TElements);
    function Element(ACol, ARow : integer) : IElement;
    procedure LoadFromResource(Elements : TElements);
  end;

implementation

uses
  System.Classes, System.SysUtils, System.Types;

{$R 'element.res' 'element.rc'}

{ TElements }

type
  TElement = class(TInterfacedObject, IElement)
  private
    FName : string;
    FNumber : integer;
    FSymbol : string;
    FWeight : double;
  public
    procedure SetName(Value : string);
    function GetName : string;
    property Name : string read GetName write SetName;

    procedure SetNumber(Value : integer);
    function GetNumber : integer;
    property Number : integer read GetNumber write SetNumber;

    procedure SetSymbol(Value : string);
    function GetSymbol : string;
    property Symbol : string read GetSymbol write SetSymbol;

    procedure SetWeight(Value : double);
    function GetWeight : double;
    property Weight : double read GetWeight write SetWeight;
  end;


function TElements.Count: integer;
begin
  result := List.Count;
end;

constructor TElements.Create;
begin
  inherited;
  List := TList<IElement>.Create;
  LoadFromResource;
end;

destructor TElements.Destroy;
begin
  List.Free;
end;

function TElements.Element(AtomicNumber: integer): IElement;
begin
  result := List.items[AtomicNumber - 1];
end;

procedure TElements.LoadFromResource;
var
  FullFile, Line : TStringList;
  I: Integer;
  e : TElement;
  rs : TResourceStream;
begin
  FullFile := TStringList.Create;
  Line := TStringList.Create;
  rs := TResourceStream.Create(HInstance, 'elements_csv', RT_RCDATA);
  try
    FullFile.LoadFromStream(rs);
    for I := 1 to FullFile.Count - 1 do
    begin
      Line.CommaText := FullFile[i];
      e := TElement.Create;
      e.Name := Line[0];
      e.Number := StrToInt(Line[1]);
      e.Symbol := Line[2];
      e.Weight := StrToFloat(Line[3]);
      List.Add(e);
    end;
  finally
    FullFile.Free;
    Line.Free;
    rs.Free;
  end;
end;

{ TElement }

function TElement.GetName: string;
begin
  result := FName;
end;

function TElement.GetNumber: integer;
begin
  result := FNumber;
end;

function TElement.GetSymbol: string;
begin
  result := FSymbol;
end;

function TElement.GetWeight: double;
begin
  result := FWeight;
end;

procedure TElement.SetName(Value: string);
begin
  FName := Value;
end;

procedure TElement.SetNumber(Value: integer);
begin
  FNumber := Value;
end;

procedure TElement.SetSymbol(Value: string);
begin
  FSymbol := Value;
end;

procedure TElement.SetWeight(Value: double);
begin
  FWeight := Value;
end;

{ TPeriodicTable }

constructor TPeriodicTable.Create(Elements: TElements);
begin
  LoadFromResource(Elements);
end;

function TPeriodicTable.Element(ACol, ARow: integer): IElement;
begin
  result := TheTable[ACol, ARow];
end;

procedure TPeriodicTable.LoadFromResource(Elements: TElements);
var
  TableRowCol, Line : TStringList;
  I: Integer;
  Num: Integer;
  Row: Integer;
  Col: Integer;
  rs: TResourceStream;
begin
  TableRowCol := TStringList.Create;
  Line := TStringList.Create;
  rs := TResourceStream.Create(HInstance, 'PeriodicTableLocations_csv', RT_RCDATA);
  TableRowCol.LoadFromStream(rs);
  for I := 1 to TableRowCol.Count - 1 do
  begin
    Line.CommaText := TableRowCol[i];
    Num := StrToInt(Line[0]);
    Row := StrToInt(Line[1]);
    Col := StrToInt(Line[2]);
    if num > elements.Count then
      Break;
    TheTable[Col, Row] := Elements.Element(Num);
  end;

  Line.Free;
  TableRowCol.Free;
  rs.Free;
end;

end.
